# Generated by Django 3.2.5 on 2021-08-08 01:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0053_characteraudit_last_update_contacts'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='charactertitle',
            name='character',
        ),
        migrations.AddField(
            model_name='characteraudit',
            name='last_update_titles',
            field=models.DateTimeField(blank=True, default=None, null=True),
        ),
        migrations.AddField(
            model_name='characterroles',
            name='titles',
            field=models.ManyToManyField(to='corptools.CharacterTitle'),
        ),
        migrations.AddField(
            model_name='charactertitle',
            name='corporation_id',
            field=models.BigIntegerField(),
            preserve_default=False,
        ),
    ]



# Generated by Django 3.2.8 on 2022-01-09 01:42

from django.db import migrations


def migrate_notifications(apps, schema_editor):
    Notification = apps.get_model('corptools', 'Notification')
    NotificationText = apps.get_model('corptools', 'NotificationText')
    print("Starting Migration!")
    print("this will look like it has hung but it is doing bulk migration of data in the background")

    def get_missing_qs():
        existing_notes = NotificationText.objects.all().values('notification_id')
        return Notification.objects.all().values(
            'notification_id').exclude(
                notification_id__in=existing_notes).distinct()

    step = 50000
    note_cnt = get_missing_qs().count()

    while note_cnt > 0:
        n = get_missing_qs().values('notification_id',
                                    'notification_text')[:step]
        obs = []
        for i in n:
            obs.append(NotificationText(notification_id=i['notification_id'],
                                        notification_text=i['notification_text']))
        c = NotificationText.objects.bulk_create(
            obs, batch_size=20000, ignore_conflicts=True)

        print(f"Migrated {len(c)} of {note_cnt}")

        note_cnt = get_missing_qs().count()

    newids = NotificationText.objects.all().values('notification_id')
    cnt = Notification.objects.all().exclude(notification_id__in=newids).count()

    if cnt > 0:
        print(
            f"Notifications have not been migrated fully, {cnt} missing notifications, please retry migrations. If Problem persists please contact the developers")


class Migration(migrations.Migration):
    atomic = False
    dependencies = [
        ('corptools', '0061_notificationtext'),
    ]

    operations = [
        migrations.RunPython(migrate_notifications)
    ]



# Generated by Django 4.2.13 on 2024-09-08 09:16

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0109_corptoolsconfiguration_disable_verification_assets_and_more'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='OrePrice',
            new_name='TypePrice',
        ),
    ]



from django.urls import include, path, re_path

from corptools.audit_views.character import (
    assets, assets_lists, clones, contacts, market, notifications, pub_data,
    roles, skills, status, wallet,
)
from corptools.audit_views.corporation import corp_list

from . import app_settings, views
from .api import api

app_name = 'corptools'

_character_ulrs = [
    path('pub/', pub_data, name='public_data'),
    path('overview/', status, name='overview'),
]

if app_settings.CT_CHAR_ASSETS_MODULE:
    _character_ulrs += [
        path('assets/', assets, name='assets'),
        re_path(r'^assets_lists/(?P<location_id>(\d)*)/$',
                assets_lists, name='assets_lists'),
    ]

if app_settings.CT_CHAR_WALLET_MODULE:
    _character_ulrs += [
        path('market/', market, name='market'),
        path('wallet/', wallet, name='wallet'),
    ]

if app_settings.CT_CHAR_ROLES_MODULE:
    _character_ulrs += [
        path('roles/', roles, name='roles'),
    ]

if app_settings.CT_CHAR_NOTIFICATIONS_MODULE:
    _character_ulrs += [
        path('notes/', notifications, name='notifications'),
    ]

if app_settings.CT_CHAR_CLONES_MODULE:
    _character_ulrs += [
        path('clones/', clones, name='clones'),
    ]

if app_settings.CT_CHAR_SKILLS_MODULE:
    _character_ulrs += [
        path('skills/', skills, name='skills'),
    ]

if app_settings.CT_CHAR_CONTACTS_MODULE:
    _character_ulrs += [
        path('contacts/', contacts, name='contacts'),
    ]

urlpatterns = [
    path('', views.corptools_menu, name='view'),
    re_path(r'^api/', api.urls),
    path(
        'r/',
        views.react_menu,
        name='react'
    ),
    re_path(
        r'^r/(?P<character_id>(\d)*)/',
        views.v3_ui_render,
        name='reactmain'
    ),
    re_path(
        'r_beta/corp',
        views.react_corp_beta,
        name='corp_beta_react'
    ),
    path(
        'r_beta/',
        views.react_menu_v4,
        name='reactbeta'
    ),
    re_path(
        r'^r_beta/(?P<character_id>(\d)*)/',
        views.react_main,
        name='reactbetamain'
    ),
    re_path(r'^check_account/(?P<character_id>(\d)*)/$',
            views.update_account, name='update_account'),
    path('admin/', views.admin, name='admin'),
    path('admin_add_pyfa_xml/', views.admin_add_pyfa_xml,
         name='admin_add_pyfa_xml'),
    path('admin_create_tasks/', views.admin_create_tasks,
         name='admin_create_tasks'),
    path('run_tasks/', views.admin_run_tasks, name='run_tasks'),
    path('char/add/', views.add_char, name='add_char'),
    re_path(r'^char/(?P<character_id>(\d)*)/', include(_character_ulrs)),
    path('corp/', include([
        re_path(r'^menu/', corp_list, name='corp_menu'),
        path('r/', views.react_corp, name='corp_react'),
        path('add/', views.add_corp, name='add_corp'),
        path('add_options/', views.add_corp_section,
             name='add_corp_options'),
        re_path(r'^dashboard/fuel', views.fuel_levels, name='fuel_dashboard'),
        re_path(r'^dashboard/metenox', views.metenox_levels,
                name='drill_dashboard'),
    ])),
]



# Generated by Django 4.2.16 on 2024-09-23 00:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('eveonline', '0017_alliance_and_corp_names_are_not_unique'),
        ('corptools', '0111_corptoolsconfiguration_disable_update_assets_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='corptoolsconfiguration',
            name='holding_corps',
            field=models.ManyToManyField(blank=True, to='eveonline.evecorporationinfo'),
        ),
    ]



from .assets import AssetsApiEndpoints
from .at_a_glance import CorpGlanceApiEndpoints
from .dashboards import DashboardApiEndpoints
from .finances import FinancesApiEndpoints
from .list import ListApiEndpoints
from .status import StatusApiEndpoints
from .structures import StructureApiEndpoints


def setup(api):
    AssetsApiEndpoints(api)
    FinancesApiEndpoints(api)
    ListApiEndpoints(api)
    StatusApiEndpoints(api)
    DashboardApiEndpoints(api)
    StructureApiEndpoints(api)
    CorpGlanceApiEndpoints(api)



import logging

from aadiscordbot.app_settings import get_all_servers
from aadiscordbot.cogs.utils.autocompletes import search_characters
from aadiscordbot.cogs.utils.decorators import (
    has_any_perm, in_channels, sender_has_perm,
)
from discord import option
from discord.embeds import Embed
from discord.ext import commands
from pendulum.datetime import DateTime

from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist

from allianceauth.eveonline.models import EveCharacter
from esi.models import Token

from corptools.models import EveItemType, MapSystem
from corptools.providers import esi

logger = logging.getLogger(__name__)


class Locator(commands.Cog):
    """
    All about users!
    """

    def __init__(self, bot):
        self.bot = bot

    def get_locate_embeds(self, char):
        alts = char.character_ownership.user.character_ownerships.all().select_related('character')

        alt_online = []
        alt_offline = []
        alt_no_token = []
        for alt in alts:
            _alt = {
                "cid": alt.character.character_id,
                "cnm": alt.character.character_name,
                "crpid": alt.character.corporation_id,
                "crpnm": alt.character.corporation_name,
                "last_online": DateTime.min,
                "online": False,
                "ship": "",
                "system": None,
                "lookup": False
            }
            scopes = [
                # Locations
                'esi-location.read_location.v1',
                'esi-location.read_online.v1',
                'esi-location.read_ship_type.v1'
            ]

            t = Token.get_token(alt.character.character_id, scopes)
            if t:
                online = esi.client.Location.get_characters_character_id_online(character_id=alt.character.character_id,
                                                                                token=t.valid_access_token()).result()
                try:
                    _alt['online'] = "**Online**" if online['online'] else "**Offline**"
                except Exception:
                    pass

                try:
                    _alt['last_online'] = online['last_logout']
                except Exception:
                    pass

                location = esi.client.Location.get_characters_character_id_location(character_id=alt.character.character_id,
                                                                                    token=t.valid_access_token()).result()
                _alt['system'] = MapSystem.objects.get(
                    system_id=location['solar_system_id']).name

                ship = esi.client.Location.get_characters_character_id_ship(character_id=alt.character.character_id,
                                                                            token=t.valid_access_token()).result()
                shp, _ = EveItemType.objects.get_or_create_from_esi(
                    ship['ship_type_id'])
                _alt['ship'] = shp
                _alt['lookup'] = True
                if online["online"]:
                    alt_online.append(_alt)
                else:
                    alt_offline.append(_alt)
            else:
                alt_no_token.append(_alt)

        out_embeds = []

        def process_list(header, alt_list):
            embeds = []
            for alt_grp in [alt_list[i:i + 10] for i in range(0, len(alt_list), 10)]:
                altstr = []
                for a in alt_grp:
                    if a['lookup']:
                        altstr.append(f"[{a['cnm']}](https://evewho.com/character/{a['cid']}) "
                                      f"*[ [{a['crpnm']}](https://evewho.com/corporation/{a['crpid']}) ]*: "
                                      f"{a['online']}"
                                      f" in [{a['system']}](https://evemaps.dotlan.net/system/{a['system'].replace(' ', '_')})"
                                      f"\n ```Currently in a {a['ship']}"
                                      f" Last Online: {a['last_online'].strftime('%Y-%m-%d %H:%M')}```")
                    else:
                        altstr.append(f"[{a['cnm']}](https://evewho.com/character/{a['cid']}) "
                                      f"*[ [{a['crpnm']}](https://evewho.com/corporation/{a['crpid']}) ]*: "
                                      f"Unknown No Tokens"
                                      f"")

                e = Embed(title=header)
                e.description = "\n".join(altstr)
                embeds.append(e)
            return embeds
        if len(alt_online):
            out_embeds += process_list("Online", alt_online)
        if len(alt_offline):
            out_embeds += process_list("Offline", alt_offline)
        if len(alt_no_token):
            out_embeds += process_list("No Tokens", alt_no_token)
        return out_embeds

    @commands.command(pass_context=True)
    @sender_has_perm('corputils.view_alliance_corpstats')
    async def locate(self, ctx):
        """
        Gets Auth data about a character
        Input: a Eve Character Name
        """
        if ctx.message.channel.id not in settings.ADMIN_DISCORD_BOT_CHANNELS:
            return await ctx.message.add_reaction(chr(0x1F44E))

        input_name = ctx.message.content[8:]

        try:
            char = EveCharacter.objects.get(character_name=input_name)
            try:
                main = char.character_ownership.user.profile.main_character
                try:
                    discord_string = "<@{}>".format(
                        char.character_ownership.user.discord.uid)
                except Exception as e:
                    logger.error(e)
                    discord_string = "unknown"

                await ctx.message.reply(f"Looing up the location of all known alts of {main} ({discord_string})\nPlease Wait...")

                embeds = self.get_locate_embeds(char)
                for e in embeds:
                    await ctx.message.reply(embed=e)
            except ObjectDoesNotExist:
                return await ctx.message.reply(f"Character **{input_name}** Unlinked in auth")
        except EveCharacter.DoesNotExist:
            return await ctx.message.reply(f"Character **{input_name}** does not exist in our Auth system")

    @commands.slash_command(name='locate', guild_ids=get_all_servers())
    @option("character", description="Search for a Character!", autocomplete=search_characters)
    async def slash_locate(self, ctx, character: str,):
        try:
            in_channels(ctx.channel.id, settings.ADMIN_DISCORD_BOT_CHANNELS)
            has_any_perm(ctx.author.id, [
                         'corputils.view_alliance_corpstats', 'corpstats.view_alliance_corpstats'])

            try:
                char = EveCharacter.objects.get(character_name=character)
                try:
                    main = char.character_ownership.user.profile.main_character
                    try:
                        discord_string = "<@{}>".format(
                            char.character_ownership.user.discord.uid)
                    except Exception as e:
                        logger.error(e)
                        discord_string = "unknown"

                    await ctx.respond(f"Looing up the location of all known alts of {main} ({discord_string})\nPlease Wait...")

                    embeds = self.get_locate_embeds(char)

                    for e in embeds:
                        await ctx.respond(embed=e)
                except ObjectDoesNotExist:
                    return await ctx.respond(f"Character **{character}** Unlinked in auth")
            except EveCharacter.DoesNotExist:
                return await ctx.respond(f"Character **{character}** does not exist in our Auth system")
        except commands.MissingPermissions as e:
            return await ctx.respond(e.missing_permissions[0], ephemeral=True)


def setup(bot):
    bot.add_cog(Locator(bot))



# Generated by Django 3.1.6 on 2021-03-23 15:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0044_auto_20210322_0805'),
    ]

    operations = [
        migrations.AlterField(
            model_name='characterasset',
            name='id',
            field=models.BigAutoField(primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='corpasset',
            name='id',
            field=models.BigAutoField(primary_key=True, serialize=False),
        ),
    ]



# Generated by Django 4.2.5 on 2023-09-16 09:41

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0091_assetsfilter_count_message_only_and_more'),
    ]

    operations = [
        migrations.CreateModel(
            name='Poco',
            fields=[
                ('id', models.AutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('office_id', models.BigIntegerField()),
                ('alliance_tax_rate', models.FloatField(default=None, null=True)),
                ('corporation_tax_rate', models.FloatField(default=None, null=True)),
                ('terrible_standing_tax_rate',
                 models.FloatField(default=None, null=True)),
                ('bad_standing_tax_rate', models.FloatField(default=None, null=True)),
                ('neutral_standing_tax_rate',
                 models.FloatField(default=None, null=True)),
                ('good_standing_tax_rate', models.FloatField(
                    default=None, null=True)),
                ('excellent_standing_tax_rate',
                 models.FloatField(default=None, null=True)),
                ('reinforce_exit_end', models.IntegerField()),
                ('reinforce_exit_start', models.IntegerField()),
                ('standing_level', models.CharField(
                    default=None, max_length=150, null=True)),
                ('allow_access_with_standings',
                 models.BooleanField(default=None, null=True)),
                ('allow_alliance_access', models.BooleanField(
                    default=None, null=True)),
                ('system_id', models.IntegerField()),
                ('name', models.CharField(blank=True,
                 default=None, max_length=150, null=True)),
                ('corporation', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE,
                 related_name='ct_poco', to='corptools.corporationaudit')),
                ('planet', models.ForeignKey(default=None, null=True,
                 on_delete=django.db.models.deletion.SET_NULL, to='corptools.mapsystemplanet')),
                ('system_name', models.ForeignKey(default=None, null=True,
                 on_delete=django.db.models.deletion.SET_NULL, to='corptools.mapsystem')),
            ],
        ),
    ]



# flake8: noqa
from datetime import timedelta

from django.contrib.auth.models import Group, User
from django.test import TestCase
from django.utils import timezone

from allianceauth.authentication.models import CharacterOwnership
from allianceauth.eveonline.models import (
    EveAllianceInfo, EveCharacter, EveCorporationInfo,
)
from allianceauth.tests.auth_utils import AuthUtils

from corptools import models as ct_models


class TestSecGroupBotFilters(TestCase):
    @classmethod
    def setUpTestData(cls):

        ct_models.CharacterAudit.objects.all().delete()
        EveCharacter.objects.all().delete()
        User.objects.all().delete()
        CharacterOwnership.objects.all().delete()
        ct_models.EveLocation.objects.all().delete()
        ct_models.MapRegion.objects.all().delete()
        ct_models.MapConstellation.objects.all().delete()
        ct_models.MapSystem.objects.all().delete()
        ct_models.MapSystem.objects.all().delete()
        ct_models.Skill.objects.all().delete()

        userids = range(1, 11)

        cls.test_group, _ = Group.objects.update_or_create(name="Test_Group")

        users = []
        characters = []
        for uid in userids:
            user = AuthUtils.create_user(f"User_{uid}")
            main_char = AuthUtils.add_main_character_2(user,
                                                       f"Main {uid}",
                                                       uid,
                                                       corp_id=1,
                                                       corp_name='Test Corp 1',
                                                       corp_ticker='TST1')
            CharacterOwnership.objects.create(
                user=user, character=main_char, owner_hash=f"main{uid}")

            characters.append(main_char)
            users.append(user)
        cls.c1t1 = ct_models.CharacterTitle.objects.create(corporation_id=1,
                                                           corporation_name='Test Corp 1',
                                                           title="Test Title1",
                                                           title_id=1)

        cls.c1t2 = ct_models.CharacterTitle.objects.create(corporation_id=1,
                                                           corporation_name='Test Corp 1',
                                                           title="Test Title2",
                                                           title_id=2)

        # add some extra characters to users in 2 corps/alliance
        for uid in range(0, 5):  # test corp 2
            character = EveCharacter.objects.create(character_name=f'Alt {uid}',
                                                    character_id=11 + uid,
                                                    corporation_name='Test Corp 2',
                                                    corporation_id=2,
                                                    corporation_ticker='TST2')
            CharacterOwnership.objects.create(character=character,
                                              user=users[uid],
                                              owner_hash=f'ownalt{11 + uid}')
            characters.append(character)

        cls.c2t1 = ct_models.CharacterTitle.objects.create(corporation_id=2,
                                                           corporation_name='Test Corp 2',
                                                           title="Test Title1",
                                                           title_id=1)

        cls.c2t2 = ct_models.CharacterTitle.objects.create(corporation_id=2,
                                                           corporation_name='Test Corp 2',
                                                           title="Test Title2",
                                                           title_id=2)

        for uid in range(5, 10):  # Test alliance 1
            character = EveCharacter.objects.create(character_name=f'Alt {uid}',
                                                    character_id=11 + uid,
                                                    corporation_name='Test Corp 3',
                                                    corporation_id=3,
                                                    corporation_ticker='TST3',
                                                    alliance_id=1,
                                                    alliance_name="Test Alliance 1",
                                                    alliance_ticker="TSTA1")
            CharacterOwnership.objects.create(character=character,
                                              user=users[uid],
                                              owner_hash=f'ownalt{11+uid}')
            characters.append(character)

        cls.alli = EveAllianceInfo.objects.create(alliance_id=1,
                                                  alliance_name="Test Alliance 1",
                                                  alliance_ticker="TSTA1",
                                                  executor_corp_id=3)

        cls.corp = EveCorporationInfo.objects.create(corporation_id=3,
                                                     corporation_name="Test Corp 3",
                                                     corporation_ticker="TST3",
                                                     member_count=10,
                                                     alliance=cls.alli,)

        # add some systems
        r1 = ct_models.MapRegion.objects.create(
            name="Test region 1",
            description="This region is junk at best, nothing is even in range",
            region_id=1
        )
        r2 = ct_models.MapRegion.objects.create(
            name="Test region 2",
            description="This region is a flood plain",
            region_id=2
        )
        r3 = ct_models.MapRegion.objects.create(
            name="Test region 3",
            description="This region is a fortress",
            region_id=3)

        c1 = ct_models.MapConstellation.objects.create(name="Test Constellation 1",
                                                       constellation_id=1,
                                                       region=r1)
        c2 = ct_models.MapConstellation.objects.create(name="Test Constellation 2",
                                                       constellation_id=2,
                                                       region=r2)
        c3 = ct_models.MapConstellation.objects.create(name="Test Constellation 3",
                                                       constellation_id=3,
                                                       region=r3)
        s1 = ct_models.MapSystem.objects.create(name="Test System 1",
                                                system_id=1,
                                                security_status=0.0,
                                                x=1,
                                                y=1,
                                                z=1,
                                                security_class="g",
                                                star_id=1,
                                                constellation=c1)
        s2 = ct_models.MapSystem.objects.create(name="Test System 2",
                                                system_id=2,
                                                security_status=0.0,
                                                x=1,
                                                y=1,
                                                z=1,
                                                security_class="g",
                                                star_id=2,
                                                constellation=c2)
        s3 = ct_models.MapSystem.objects.create(name="Test System 3",
                                                system_id=3,
                                                security_status=0.0,
                                                x=1,
                                                y=1,
                                                z=1,
                                                security_class="g",
                                                star_id=3,
                                                constellation=c3)

        audits = []
        for uid in range(0, 17):  # 4,5,6,7
            audits.append(ct_models.CharacterAudit.objects.create(character=characters[uid],
                                                                  last_update_assets=timezone.now(),
                                                                  last_update_clones=timezone.now(),
                                                                  last_update_pub_data=timezone.now(),
                                                                  last_update_skill_que=timezone.now(),
                                                                  last_update_skills=timezone.now(),
                                                                  last_update_wallet=timezone.now(),
                                                                  last_update_orders=timezone.now(),
                                                                  last_update_notif=timezone.now(),
                                                                  last_update_roles=timezone.now(),
                                                                  last_update_mails=timezone.now(),))
        audits.append(ct_models.CharacterAudit.objects.create(
            character=characters[17]))

        skg1 = ct_models.EveItemGroup.objects.create(group_id=1,
                                                     name="TestGroup")
        sk1 = ct_models.EveItemType.objects.create(type_id=1,
                                                   name="Skill 1",
                                                   published=True,
                                                   group=skg1)
        sk2 = ct_models.EveItemType.objects.create(type_id=2,
                                                   name="Skill 2",
                                                   published=True,
                                                   group=skg1)
        sk3 = ct_models.EveItemType.objects.create(type_id=3,
                                                   name="Skill 3",
                                                   published=True,
                                                   group=skg1)
        sk4 = ct_models.EveItemType.objects.create(type_id=4,
                                                   name="Skill 4",
                                                   published=True,
                                                   group=skg1)
        sk5 = ct_models.EveItemType.objects.create(type_id=5,
                                                   name="Skill 5",
                                                   published=True,
                                                   group=skg1)
        skc1 = ct_models.EveItemCategory.objects.create(category_id=1,
                                                        name="TestCat")
        skg2 = ct_models.EveItemGroup.objects.create(group_id=2,
                                                     name="TestGroup",
                                                     category=skc1)
        a1 = ct_models.EveItemType.objects.create(type_id=10,
                                                  name="Asset 1",
                                                  published=True,
                                                  group=skg2)
        a2 = ct_models.EveItemType.objects.create(type_id=11,
                                                  name="Asset 2",
                                                  published=True)
        a3 = ct_models.EveItemType.objects.create(type_id=12,
                                                  name="Asset 3",
                                                  published=True)
        a4 = ct_models.EveItemType.objects.create(type_id=13,
                                                  name="Asset 4",
                                                  published=True)
        a5 = ct_models.EveItemType.objects.create(type_id=14,
                                                  name="Asset 5",
                                                  published=True)

        l1 = ct_models.EveLocation.objects.create(location_id=1,
                                                  location_name="Test Location 1",
                                                  system=s1)
        l2 = ct_models.EveLocation.objects.create(location_id=2,
                                                  location_name="Test Location 2",
                                                  system=s2)
        l3 = ct_models.EveLocation.objects.create(location_id=3,
                                                  location_name="Test Location 3",
                                                  system=s3)

        ct_models.CharacterLocation.objects.create(
            character=audits[0],
            current_location=l1,
            current_ship=a1,
            current_ship_name=f"Test Ship {a1.type_id} {a1.name}"
        )

        ct_models.CharacterLocation.objects.create(
            character=audits[1],
            current_location=l1,
            current_ship=a1,
            current_ship_name=f"Test Ship {a1.type_id} {a1.name}"
        )

        ct_models.CharacterLocation.objects.create(
            character=audits[2],
            current_location=l2,
            current_ship=a1,
            current_ship_name=f"Test Ship {a1.type_id} {a1.name}"
        )

        ct_models.CharacterLocation.objects.create(
            character=audits[3],
            current_location=l2,
            current_ship=a2,
            current_ship_name=f"Test Ship {a2.type_id} {a2.name}"
        )

        ct_models.CharacterLocation.objects.create(
            character=audits[4],
            current_location=l3,
            current_ship=a2,
            current_ship_name=f"Test Ship {a1.type_id} {a1.name}"
        )

        ct_models.CharacterLocation.objects.create(
            character=audits[5],
            current_location=l3,
            current_ship=a2,
            current_ship_name=f"Test Ship {a1.type_id} {a1.name}"
        )

        ct_models.CharacterAsset.objects.create(item_id=0,
                                                location_flag="test",
                                                location_id=0,
                                                quantity=1,
                                                singleton=True,
                                                location_type="test",
                                                type_id=10,
                                                type_name=a1,
                                                location_name=l1,
                                                character=audits[0],
                                                blueprint_copy=False)

        ct_models.CharacterAsset.objects.create(item_id=0,
                                                location_flag="test",
                                                location_id=0,
                                                quantity=1,
                                                singleton=True,
                                                location_type="test",
                                                type_id=10,
                                                type_name=a1,
                                                location_name=l1,
                                                character=audits[2],
                                                blueprint_copy=False)

        ct_models.CharacterAsset.objects.create(item_id=0,
                                                location_flag="test",
                                                location_id=0,
                                                quantity=1,
                                                singleton=True,
                                                location_type="test",
                                                type_id=10,
                                                type_name=a1,
                                                location_name=l1,
                                                character=audits[15],
                                                blueprint_copy=False)

        ct_models.CharacterAsset.objects.create(item_id=0,
                                                location_flag="test",
                                                location_id=0,
                                                quantity=1,
                                                singleton=True,
                                                location_type="test",
                                                type_id=10,
                                                type_name=a1,
                                                location_name=l1,
                                                character=audits[17],
                                                blueprint_copy=False)

        ct_models.CharacterAsset.objects.create(item_id=0,
                                                location_flag="test",
                                                location_id=0,
                                                quantity=1,
                                                singleton=True,
                                                location_type="test",
                                                type_id=11,
                                                type_name=a2,
                                                location_name=l1,
                                                character=audits[4],
                                                blueprint_copy=False)

        ct_models.CharacterAsset.objects.create(item_id=0,
                                                location_flag="test",
                                                location_id=0,
                                                quantity=1,
                                                singleton=True,
                                                location_type="test",
                                                type_id=11,
                                                type_name=a2,
                                                location_name=l1,
                                                character=audits[6],
                                                blueprint_copy=False)

        ct_models.CharacterAsset.objects.create(item_id=0,
                                                location_flag="test",
                                                location_id=0,
                                                quantity=1,
                                                singleton=True,
                                                location_type="test",
                                                type_id=11,
                                                type_name=a2,
                                                location_name=l1,
                                                character=audits[15],
                                                blueprint_copy=False)

        ct_models.CharacterAsset.objects.create(item_id=0,
                                                location_flag="test",
                                                location_id=0,
                                                quantity=1,
                                                singleton=True,
                                                location_type="test",
                                                type_id=11,
                                                type_name=a2,
                                                location_name=l1,
                                                character=audits[17],
                                                blueprint_copy=False)

        ct_models.CharacterAsset.objects.create(item_id=0,
                                                location_flag="test",
                                                location_id=0,
                                                quantity=1,
                                                singleton=True,
                                                location_type="test",
                                                type_id=11,
                                                type_name=a2,
                                                location_name=l2,
                                                character=audits[1],
                                                blueprint_copy=False)

        ct_models.CharacterAsset.objects.create(item_id=0,
                                                location_flag="test",
                                                location_id=0,
                                                quantity=1,
                                                singleton=True,
                                                location_type="test",
                                                type_id=11,
                                                type_name=a2,
                                                location_name=l2,
                                                character=audits[3],
                                                blueprint_copy=False)

        ct_models.CharacterAsset.objects.create(item_id=0,
                                                location_flag="test",
                                                location_id=0,
                                                quantity=1,
                                                singleton=True,
                                                location_type="test",
                                                type_id=11,
                                                type_name=a2,
                                                location_name=l2,
                                                character=audits[14],
                                                blueprint_copy=False)

        ct_models.CharacterAsset.objects.create(item_id=0,
                                                location_flag="test",
                                                location_id=0,
                                                quantity=1,
                                                singleton=True,
                                                location_type="test",
                                                type_id=11,
                                                type_name=a2,
                                                location_name=l2,
                                                character=audits[16],
                                                blueprint_copy=False)

        ct_models.SkillList.objects.create(name="Test Skills 1",
                                           skill_list='{"Skill 1":"5"}')
        ct_models.SkillList.objects.create(name="Test Skills 2",
                                           skill_list='{"Skill 2":"1"}')

        for a in audits:
            ct_models.Skill.objects.create(character=a,
                                           skill_id=sk2.type_id,
                                           skill_name=sk2,
                                           active_skill_level=0,
                                           trained_skill_level=5,
                                           skillpoints_in_skill=500)

        ct_models.Skill.objects.create(character=audits[0],
                                       skill_id=sk1.type_id,
                                       skill_name=sk1,
                                       active_skill_level=5,
                                       trained_skill_level=5,
                                       skillpoints_in_skill=500)

        ct_models.Skill.objects.create(character=audits[9],
                                       skill_id=sk1.type_id,
                                       skill_name=sk1,
                                       active_skill_level=5,
                                       trained_skill_level=5,
                                       skillpoints_in_skill=500)

        u1 = ct_models.CharacterRoles.objects.create(
            character=audits[0], director=True, personnel_manager=True)

        ct_models.CharacterRoles.objects.create(
            character=audits[16], director=True, personnel_manager=True)

        u1.titles.add(cls.c1t1)

        u15 = ct_models.CharacterRoles.objects.create(
            character=audits[15], personnel_manager=True)
        u15.titles.add(cls.c2t1)

        u16 = ct_models.CharacterRoles.objects.create(
            character=audits[17], personnel_manager=True)
        u16.titles.add(cls.c2t1)

        # print("**************************")
        # print(u1.character.character.character_ownership.user.id)
        # print(u15.character.character.character_ownership.user.id)
        # print(u16.character.character.character_ownership.user.id)
        # print("**************************")

        ct_models.Skill.objects.filter(
            character=audits[0]).update(active_skill_level=5)

    def test_user_loaded_fully(self):
        _filter = ct_models.FullyLoadedFilter.objects.create(name="Fully Loaded Test",
                                                             description="Something to tell user")
        users = {}
        for user in ct_models.CharacterAudit.objects.all():
            users[user.character.character_ownership.user.id] = None

        tests = {}
        for k, u in users.items():
            tests[k] = _filter.process_filter(User.objects.get(id=k))

        self.assertTrue(tests[1])
        self.assertTrue(tests[2])
        self.assertTrue(tests[3])
        self.assertTrue(tests[4])
        self.assertTrue(tests[5])
        self.assertTrue(tests[6])
        self.assertTrue(tests[7])
        self.assertFalse(tests[8])
        self.assertFalse(tests[9])
        self.assertFalse(tests[10])

        tests = {}
        tests = _filter.audit_filter(
            User.objects.filter(id__in=list(users.keys())))
        # print(tests)

        self.assertTrue(tests[1]["check"])
        self.assertTrue(tests[2]["check"])
        self.assertTrue(tests[3]["check"])
        self.assertTrue(tests[4]["check"])
        self.assertTrue(tests[5]["check"])
        self.assertTrue(tests[6]["check"])
        self.assertTrue(tests[7]["check"])
        self.assertFalse(tests[8]["check"])
        self.assertFalse(tests[9]["check"])
        self.assertFalse(tests[10]["check"])

    def test_user_loaded_fully_reverse(self):
        _filter = ct_models.FullyLoadedFilter.objects.create(name="Fully Loaded Test",
                                                             description="Something to tell user",
                                                             reversed_logic=True)
        users = {}
        for user in ct_models.CharacterAudit.objects.all():
            users[user.character.character_ownership.user.id] = None

        tests = {}
        for k, u in users.items():
            tests[k] = _filter.process_filter(User.objects.get(id=k))
        # print(tests)
        self.assertFalse(tests[1])
        self.assertFalse(tests[2])
        self.assertFalse(tests[3])
        self.assertFalse(tests[4])
        self.assertFalse(tests[5])
        self.assertFalse(tests[6])
        self.assertFalse(tests[7])
        self.assertTrue(tests[8])
        self.assertTrue(tests[9])
        self.assertTrue(tests[10])

        tests = {}
        tests = _filter.audit_filter(
            User.objects.filter(id__in=list(users.keys())))
        # print(tests)
        self.assertFalse(tests[1]["check"])
        self.assertFalse(tests[2]["check"])
        self.assertFalse(tests[3]["check"])
        self.assertFalse(tests[4]["check"])
        self.assertFalse(tests[5]["check"])
        self.assertFalse(tests[6]["check"])
        self.assertFalse(tests[7]["check"])
        self.assertTrue(tests[8]["check"])
        self.assertTrue(tests[9]["check"])
        self.assertTrue(tests[10]["check"])

    def test_user_assets_no_loc(self):
        _filter = ct_models.AssetsFilter.objects.create(name="Assets Test",
                                                        description="Something to tell user")
        _filter.types.add(ct_models.EveItemType.objects.get(type_id=10))

        users = {}
        for user in ct_models.CharacterAudit.objects.all():
            users[user.character.character_ownership.user.id] = None

        tests = {}
        for k, u in users.items():
            tests[k] = _filter.process_filter(User.objects.get(id=k))

        self.assertTrue(tests[1])
        self.assertFalse(tests[2])
        self.assertTrue(tests[3])
        self.assertFalse(tests[4])
        self.assertFalse(tests[5])
        self.assertTrue(tests[6])
        self.assertFalse(tests[7])
        self.assertTrue(tests[8])
        self.assertFalse(tests[9])
        self.assertFalse(tests[10])

    def test_user_assets_no_loc_cat(self):
        _filter = ct_models.AssetsFilter.objects.create(name="Assets Test",
                                                        description="Something to tell user")
        _filter.categories.add(
            ct_models.EveItemCategory.objects.get(category_id=1))

        users = {}
        for user in ct_models.CharacterAudit.objects.all():
            users[user.character.character_ownership.user.id] = None

        tests = {}
        for k, u in users.items():
            tests[k] = _filter.process_filter(User.objects.get(id=k))

        self.assertTrue(tests[1])
        self.assertFalse(tests[2])
        self.assertTrue(tests[3])
        self.assertFalse(tests[4])
        self.assertFalse(tests[5])
        self.assertTrue(tests[6])
        self.assertFalse(tests[7])
        self.assertTrue(tests[8])
        self.assertFalse(tests[9])
        self.assertFalse(tests[10])

    def test_user_assets_no_loc_grp(self):
        _filter = ct_models.AssetsFilter.objects.create(name="Assets Test",
                                                        description="Something to tell user")
        _filter.groups.add(ct_models.EveItemGroup.objects.get(group_id=2))

        users = {}
        for user in ct_models.CharacterAudit.objects.all():
            users[user.character.character_ownership.user.id] = None

        tests = {}
        for k, u in users.items():
            tests[k] = _filter.process_filter(User.objects.get(id=k))

        self.assertTrue(tests[1])
        self.assertFalse(tests[2])
        self.assertTrue(tests[3])
        self.assertFalse(tests[4])
        self.assertFalse(tests[5])
        self.assertTrue(tests[6])
        self.assertFalse(tests[7])
        self.assertTrue(tests[8])
        self.assertFalse(tests[9])
        self.assertFalse(tests[10])

    def test_user_assets_loc(self):
        _filter = ct_models.AssetsFilter.objects.create(name="Assets Test",
                                                        description="Something to tell user")
        _filter.types.add(ct_models.EveItemType.objects.get(type_id=11))
        _filter.systems.add(
            ct_models.MapSystem.objects.get(name="Test System 1"))

        users = {}
        for user in ct_models.CharacterAudit.objects.all():
            users[user.character.character_ownership.user.id] = None

        tests = {}
        for k, u in users.items():
            tests[k] = _filter.process_filter(User.objects.get(id=k))

        self.assertFalse(tests[1])
        self.assertFalse(tests[2])
        self.assertFalse(tests[3])
        self.assertFalse(tests[4])
        self.assertTrue(tests[5])
        self.assertTrue(tests[6])
        self.assertTrue(tests[7])
        self.assertTrue(tests[8])
        self.assertFalse(tests[9])
        self.assertFalse(tests[10])

    def test_user_assets_loc_con(self):
        _filter = ct_models.AssetsFilter.objects.create(name="Assets Test",
                                                        description="Something to tell user")
        _filter.types.add(ct_models.EveItemType.objects.get(type_id=11))
        _filter.constellations.add(
            ct_models.MapConstellation.objects.get(name="Test Constellation 1"))

        users = {}
        for user in ct_models.CharacterAudit.objects.all():
            users[user.character.character_ownership.user.id] = None

        tests = {}
        for k, u in users.items():
            tests[k] = _filter.process_filter(User.objects.get(id=k))

        self.assertFalse(tests[1])
        self.assertFalse(tests[2])
        self.assertFalse(tests[3])
        self.assertFalse(tests[4])
        self.assertTrue(tests[5])
        self.assertTrue(tests[6])
        self.assertTrue(tests[7])
        self.assertTrue(tests[8])
        self.assertFalse(tests[9])
        self.assertFalse(tests[10])

    def test_user_assets_loc_reg(self):
        _filter = ct_models.AssetsFilter.objects.create(name="Assets Test",
                                                        description="Something to tell user")
        _filter.types.add(ct_models.EveItemType.objects.get(type_id=11))
        _filter.regions.add(
            ct_models.MapRegion.objects.get(name="Test region 1"))

        users = {}
        for user in ct_models.CharacterAudit.objects.all():
            users[user.character.character_ownership.user.id] = None

        tests = {}
        for k, u in users.items():
            tests[k] = _filter.process_filter(User.objects.get(id=k))

        self.assertFalse(tests[1])
        self.assertFalse(tests[2])
        self.assertFalse(tests[3])
        self.assertFalse(tests[4])
        self.assertTrue(tests[5])
        self.assertTrue(tests[6])
        self.assertTrue(tests[7])
        self.assertTrue(tests[8])
        self.assertFalse(tests[9])
        self.assertFalse(tests[10])

    def test_user_assets_loc_2(self):
        _filter = ct_models.AssetsFilter.objects.create(name="Assets Test",
                                                        description="Something to tell user")
        _filter.types.add(ct_models.EveItemType.objects.get(type_id=11))
        _filter.systems.add(
            ct_models.MapSystem.objects.get(name="Test System 2"))

        users = {}
        for user in ct_models.CharacterAudit.objects.all():
            users[user.character.character_ownership.user.id] = None

        tests = {}
        for k, u in users.items():
            tests[k] = _filter.process_filter(User.objects.get(id=k))

        self.assertFalse(tests[1])
        self.assertTrue(tests[2])
        self.assertFalse(tests[3])
        self.assertTrue(tests[4])
        self.assertTrue(tests[5])
        self.assertFalse(tests[6])
        self.assertTrue(tests[7])
        self.assertFalse(tests[8])
        self.assertFalse(tests[9])

    def test_user_skill_lists_alpha(self):
        _filter = ct_models.Skillfilter.objects.create(name="Skills Test",
                                                       description="Something to tell user")
        _filter.required_skill_lists.add(
            ct_models.SkillList.objects.get(name="Test Skills 2"))

        users = {}
        for user in ct_models.CharacterAudit.objects.all():
            users[user.character.character_ownership.user.id] = None

        tests = {}
        for k, u in users.items():
            tests[k] = _filter.process_filter(User.objects.get(id=k))
        # print("**********")
        # print(tests)
        self.assertTrue(tests[1])
        self.assertFalse(tests[2])
        self.assertFalse(tests[3])
        self.assertFalse(tests[4])
        self.assertFalse(tests[5])
        self.assertFalse(tests[6])
        self.assertFalse(tests[7])
        self.assertFalse(tests[8])
        self.assertFalse(tests[9])
        self.assertFalse(tests[10])

    def test_user_skill_lists_req_one(self):
        _filter = ct_models.Skillfilter.objects.create(name="Skills Test",
                                                       description="Something to tell user")
        _filter.single_req_skill_lists.add(
            ct_models.SkillList.objects.get(name="Test Skills 2"))
        _filter.required_skill_lists.add(
            ct_models.SkillList.objects.get(name="Test Skills 1"))

        users = {}
        for user in ct_models.CharacterAudit.objects.all():
            users[user.character.character_ownership.user.id] = None

        tests = {}
        for k, u in users.items():
            tests[k] = _filter.process_filter(User.objects.get(id=k))
        # print("**********")
        # print(tests)
        self.assertTrue(tests[1])
        self.assertFalse(tests[2])
        self.assertFalse(tests[3])
        self.assertFalse(tests[4])
        self.assertFalse(tests[5])
        self.assertFalse(tests[6])
        self.assertFalse(tests[7])
        self.assertFalse(tests[8])
        self.assertFalse(tests[9])
        self.assertFalse(tests[10])

    def test_user_skill_lists_audit_req_one(self):
        _filter = ct_models.Skillfilter.objects.create(name="Skills Test",
                                                       description="Something to tell user")
        _filter.single_req_skill_lists.add(
            ct_models.SkillList.objects.get(name="Test Skills 2"))
        _filter.required_skill_lists.add(
            ct_models.SkillList.objects.get(name="Test Skills 1"))

        users = []
        for user in ct_models.CharacterAudit.objects.all():
            users.append(user.character.character_ownership.user.id)

        tests = _filter.audit_filter(User.objects.filter(id__in=users))
        # print("**********")
        # print(tests)
        self.assertTrue(tests[1]['check'])
        self.assertFalse(tests[2]['check'])
        self.assertFalse(tests[3]['check'])
        self.assertFalse(tests[4]['check'])
        self.assertFalse(tests[5]['check'])
        self.assertFalse(tests[6]['check'])
        self.assertFalse(tests[7]['check'])
        self.assertFalse(tests[8]['check'])
        self.assertFalse(tests[9]['check'])
        self.assertFalse(tests[10]['check'])

        # run again and confirm cache
        tests = _filter.audit_filter(User.objects.filter(id__in=users))
        # print("**********")
        # print(tests)
        self.assertTrue(tests[1]['check'])
        self.assertFalse(tests[2]['check'])
        self.assertFalse(tests[3]['check'])
        self.assertFalse(tests[4]['check'])
        self.assertFalse(tests[5]['check'])
        self.assertFalse(tests[6]['check'])
        self.assertFalse(tests[7]['check'])
        self.assertFalse(tests[8]['check'])
        self.assertFalse(tests[9]['check'])
        self.assertFalse(tests[10]['check'])

    def test_user_skill_lists_omega(self):
        _filter = ct_models.Skillfilter.objects.create(name="Skills Test",
                                                       description="Something to tell user")
        _filter.required_skill_lists.add(
            ct_models.SkillList.objects.get(name="Test Skills 1"))

        users = {}
        for user in ct_models.CharacterAudit.objects.all():
            users[user.character.character_ownership.user.id] = None

        tests = {}
        for k, u in users.items():
            tests[k] = _filter.process_filter(User.objects.get(id=k))
        # print("**********")
        # print(tests)
        self.assertTrue(tests[1])
        self.assertFalse(tests[2])
        self.assertFalse(tests[3])
        self.assertFalse(tests[4])
        self.assertFalse(tests[5])
        self.assertFalse(tests[6])
        self.assertFalse(tests[7])
        self.assertFalse(tests[8])
        self.assertFalse(tests[9])
        self.assertTrue(tests[10])

    def test_user_skill_lists_audit_omega(self):
        _filter = ct_models.Skillfilter.objects.create(name="Skills Test",
                                                       description="Something to tell user")
        _filter.required_skill_lists.add(
            ct_models.SkillList.objects.get(name="Test Skills 1"))

        users = []
        for user in ct_models.CharacterAudit.objects.all():
            users.append(user.character.character_ownership.user.id)

        tests = _filter.audit_filter(User.objects.filter(id__in=users))
        # print("**********")
        # print(tests)
        self.assertTrue(tests[1]['check'])
        self.assertFalse(tests[2]['check'])
        self.assertFalse(tests[3]['check'])
        self.assertFalse(tests[4]['check'])
        self.assertFalse(tests[5]['check'])
        self.assertFalse(tests[6]['check'])
        self.assertFalse(tests[7]['check'])
        self.assertFalse(tests[8]['check'])
        self.assertFalse(tests[9]['check'])
        self.assertTrue(tests[10]['check'])

        # run again and confirm cache
        tests = _filter.audit_filter(User.objects.filter(id__in=users))
        # print("**********")
        # print(tests)
        self.assertTrue(tests[1]['check'])
        self.assertFalse(tests[2]['check'])
        self.assertFalse(tests[3]['check'])
        self.assertFalse(tests[4]['check'])
        self.assertFalse(tests[5]['check'])
        self.assertFalse(tests[6]['check'])
        self.assertFalse(tests[7]['check'])
        self.assertFalse(tests[8]['check'])
        self.assertFalse(tests[9]['check'])
        self.assertTrue(tests[10]['check'])

    def test_user_has_roles_director(self):
        _filter = ct_models.Rolefilter.objects.create(name="roles Test",
                                                      description="Something to tell user")
        _filter.has_director = True

        users = []
        for user in ct_models.CharacterAudit.objects.all():
            users.append(user.character.character_ownership.user.id)

        tests = _filter.audit_filter(User.objects.filter(id__in=users))
        # print("**********")
        # print(tests)
        self.assertTrue(tests[1]['check'])
        self.assertFalse(tests[2]['check'])
        self.assertFalse(tests[3]['check'])
        self.assertFalse(tests[4]['check'])
        self.assertFalse(tests[5]['check'])
        self.assertFalse(tests[6]['check'])
        self.assertTrue(tests[7]['check'])
        self.assertFalse(tests[8]['check'])
        self.assertFalse(tests[9]['check'])
        self.assertFalse(tests[10]['check'])

        # run again and confirm cache
        tests = _filter.audit_filter(User.objects.filter(id__in=users))
        # print("**********")
        # print(tests)
        self.assertTrue(tests[1]['check'])
        self.assertFalse(tests[2]['check'])
        self.assertFalse(tests[3]['check'])
        self.assertFalse(tests[4]['check'])
        self.assertFalse(tests[5]['check'])
        self.assertFalse(tests[6]['check'])
        self.assertTrue(tests[7]['check'])
        self.assertFalse(tests[8]['check'])
        self.assertFalse(tests[9]['check'])
        self.assertFalse(tests[10]['check'])

    def test_user_has_roles_director_main_only(self):
        _filter = ct_models.Rolefilter.objects.create(name="roles Test",
                                                      description="Something to tell user")
        _filter.has_director = True
        _filter.main_only = True

        users = []
        for user in ct_models.CharacterAudit.objects.all():
            users.append(user.character.character_ownership.user.id)

        tests = _filter.audit_filter(User.objects.filter(id__in=users))
        # print("**********")
        # print(tests)
        self.assertTrue(tests[1]['check'])
        self.assertFalse(tests[2]['check'])
        self.assertFalse(tests[3]['check'])
        self.assertFalse(tests[4]['check'])
        self.assertFalse(tests[5]['check'])
        self.assertFalse(tests[6]['check'])
        self.assertFalse(tests[7]['check'])
        self.assertFalse(tests[8]['check'])
        self.assertFalse(tests[9]['check'])
        self.assertFalse(tests[10]['check'])

        # run again and confirm cache
        tests = _filter.audit_filter(User.objects.filter(id__in=users))
        # print("**********")
        # print(tests)
        self.assertTrue(tests[1]['check'])
        self.assertFalse(tests[2]['check'])
        self.assertFalse(tests[3]['check'])
        self.assertFalse(tests[4]['check'])
        self.assertFalse(tests[5]['check'])
        self.assertFalse(tests[6]['check'])
        self.assertFalse(tests[7]['check'])
        self.assertFalse(tests[8]['check'])
        self.assertFalse(tests[9]['check'])
        self.assertFalse(tests[10]['check'])

    def test_user_has_roles_personel(self):
        _filter = ct_models.Rolefilter.objects.create(name="roles Test",
                                                      description="Something to tell user")
        _filter.has_personnel_manager = True

        users = []
        for user in ct_models.CharacterAudit.objects.all():
            users.append(user.character.character_ownership.user.id)

        tests = _filter.audit_filter(User.objects.filter(id__in=users))
        # print("**********")
        # print(tests)
        self.assertTrue(tests[1]['check'])
        self.assertFalse(tests[2]['check'])
        self.assertFalse(tests[3]['check'])
        self.assertFalse(tests[4]['check'])
        self.assertFalse(tests[5]['check'])
        self.assertTrue(tests[6]['check'])
        self.assertTrue(tests[7]['check'])
        self.assertTrue(tests[8]['check'])
        self.assertFalse(tests[9]['check'])
        self.assertFalse(tests[10]['check'])

        # run again and confirm cache
        tests = _filter.audit_filter(User.objects.filter(id__in=users))
        # print("**********")
        # print(tests)
        self.assertTrue(tests[1]['check'])
        self.assertFalse(tests[2]['check'])
        self.assertFalse(tests[3]['check'])
        self.assertFalse(tests[4]['check'])
        self.assertFalse(tests[5]['check'])
        self.assertTrue(tests[6]['check'])
        self.assertTrue(tests[7]['check'])
        self.assertTrue(tests[8]['check'])
        self.assertFalse(tests[9]['check'])
        self.assertFalse(tests[10]['check'])

    def test_user_has_roles_personel_main_only(self):
        _filter = ct_models.Rolefilter.objects.create(name="roles Test",
                                                      description="Something to tell user")
        _filter.has_personnel_manager = True
        _filter.main_only = True

        users = []
        for user in ct_models.CharacterAudit.objects.all():
            users.append(user.character.character_ownership.user.id)

        tests = _filter.audit_filter(User.objects.filter(id__in=users))
        # print("**********")
        # print(tests)
        self.assertTrue(tests[1]['check'])
        self.assertFalse(tests[2]['check'])
        self.assertFalse(tests[3]['check'])
        self.assertFalse(tests[4]['check'])
        self.assertFalse(tests[5]['check'])
        self.assertFalse(tests[6]['check'])
        self.assertFalse(tests[7]['check'])
        self.assertFalse(tests[8]['check'])
        self.assertFalse(tests[9]['check'])
        self.assertFalse(tests[10]['check'])

        # run again and confirm cache
        tests = _filter.audit_filter(User.objects.filter(id__in=users))
        # print("**********")
        # print(tests)
        self.assertTrue(tests[1]['check'])
        self.assertFalse(tests[2]['check'])
        self.assertFalse(tests[3]['check'])
        self.assertFalse(tests[4]['check'])
        self.assertFalse(tests[5]['check'])
        self.assertFalse(tests[6]['check'])
        self.assertFalse(tests[7]['check'])
        self.assertFalse(tests[8]['check'])
        self.assertFalse(tests[9]['check'])
        self.assertFalse(tests[10]['check'])

    def test_user_has_roles_personel_alli_filtered(self):
        _filter = ct_models.Rolefilter.objects.create(name="roles Test",
                                                      description="Something to tell user",
                                                      )
        _filter.has_personnel_manager = True
        _filter.alliances_filter.add(self.alli)

        users = []
        for user in ct_models.CharacterAudit.objects.all():
            users.append(user.character.character_ownership.user.id)

        tests = _filter.audit_filter(User.objects.filter(id__in=users))
        # print("**********")
        # print(tests)
        self.assertFalse(tests[1]['check'])
        self.assertFalse(tests[2]['check'])
        self.assertFalse(tests[3]['check'])
        self.assertFalse(tests[4]['check'])
        self.assertFalse(tests[5]['check'])
        self.assertTrue(tests[6]['check'])
        self.assertTrue(tests[7]['check'])
        self.assertTrue(tests[8]['check'])
        self.assertFalse(tests[9]['check'])
        self.assertFalse(tests[10]['check'])

        # run again and confirm cache
        tests = _filter.audit_filter(User.objects.filter(id__in=users))
        # print("**********")
        # print(tests)
        self.assertFalse(tests[1]['check'])
        self.assertFalse(tests[2]['check'])
        self.assertFalse(tests[3]['check'])
        self.assertFalse(tests[4]['check'])
        self.assertFalse(tests[5]['check'])
        self.assertTrue(tests[6]['check'])
        self.assertTrue(tests[7]['check'])
        self.assertTrue(tests[8]['check'])
        self.assertFalse(tests[9]['check'])
        self.assertFalse(tests[10]['check'])

    def test_user_has_roles_personel_corp_filtered(self):
        _filter = ct_models.Rolefilter.objects.create(name="roles Test",
                                                      description="Something to tell user",
                                                      )
        _filter.has_personnel_manager = True
        _filter.corps_filter.add(self.corp)

        users = []
        for user in ct_models.CharacterAudit.objects.all():
            users.append(user.character.character_ownership.user.id)

        tests = _filter.audit_filter(User.objects.filter(id__in=users))
        # print("**********")
        # print(tests)
        self.assertFalse(tests[1]['check'])
        self.assertFalse(tests[2]['check'])
        self.assertFalse(tests[3]['check'])
        self.assertFalse(tests[4]['check'])
        self.assertFalse(tests[5]['check'])
        self.assertTrue(tests[6]['check'])
        self.assertTrue(tests[7]['check'])
        self.assertTrue(tests[8]['check'])
        self.assertFalse(tests[9]['check'])
        self.assertFalse(tests[10]['check'])

        # run again and confirm cache
        tests = _filter.audit_filter(User.objects.filter(id__in=users))
        # print("**********")
        # print(tests)
        self.assertFalse(tests[1]['check'])
        self.assertFalse(tests[2]['check'])
        self.assertFalse(tests[3]['check'])
        self.assertFalse(tests[4]['check'])
        self.assertFalse(tests[5]['check'])
        self.assertTrue(tests[6]['check'])
        self.assertTrue(tests[7]['check'])
        self.assertTrue(tests[8]['check'])
        self.assertFalse(tests[9]['check'])
        self.assertFalse(tests[10]['check'])

    def test_user_has_title_1(self):
        _filter = ct_models.Titlefilter.objects.create(name="Title Test1",
                                                       description="Something to tell user",
                                                       titles=self.c1t1)

        users = []
        for user in ct_models.CharacterAudit.objects.all():
            users.append(user.character.character_ownership.user.id)

        tests = _filter.audit_filter(User.objects.filter(id__in=users))
        # print("**********")
        # print(tests)
        self.assertTrue(tests[1]['check'])
        self.assertFalse(tests[2]['check'])
        self.assertFalse(tests[3]['check'])
        self.assertFalse(tests[4]['check'])
        self.assertFalse(tests[5]['check'])
        self.assertFalse(tests[6]['check'])
        self.assertFalse(tests[7]['check'])
        self.assertFalse(tests[8]['check'])
        self.assertFalse(tests[9]['check'])
        self.assertFalse(tests[10]['check'])

        # run again and confirm cache
        tests = _filter.audit_filter(User.objects.filter(id__in=users))
        # print("**********")
        # print(tests)
        self.assertTrue(tests[1]['check'])
        self.assertFalse(tests[2]['check'])
        self.assertFalse(tests[3]['check'])
        self.assertFalse(tests[4]['check'])
        self.assertFalse(tests[5]['check'])
        self.assertFalse(tests[6]['check'])
        self.assertFalse(tests[7]['check'])
        self.assertFalse(tests[8]['check'])
        self.assertFalse(tests[9]['check'])
        self.assertFalse(tests[10]['check'])

    def test_user_has_title_2(self):
        _filter = ct_models.Titlefilter.objects.create(name="Title Test2",
                                                       description="Something to tell user",
                                                       titles=self.c2t1)

        users = []
        for user in ct_models.CharacterAudit.objects.all():
            users.append(user.character.character_ownership.user.id)

        tests = _filter.audit_filter(User.objects.filter(id__in=users))
        # print("**********")
        # print(tests)
        self.assertFalse(tests[1]['check'])
        self.assertFalse(tests[2]['check'])
        self.assertFalse(tests[3]['check'])
        self.assertFalse(tests[4]['check'])
        self.assertFalse(tests[5]['check'])
        self.assertTrue(tests[6]['check'])
        self.assertFalse(tests[7]['check'])
        self.assertTrue(tests[8]['check'])
        self.assertFalse(tests[9]['check'])
        self.assertFalse(tests[10]['check'])

        # run again and confirm cache
        tests = _filter.audit_filter(User.objects.filter(id__in=users))
        # print("**********")
        # print(tests)
        self.assertFalse(tests[1]['check'])
        self.assertFalse(tests[2]['check'])
        self.assertFalse(tests[3]['check'])
        self.assertFalse(tests[4]['check'])
        self.assertFalse(tests[5]['check'])
        self.assertTrue(tests[6]['check'])
        self.assertFalse(tests[7]['check'])
        self.assertTrue(tests[8]['check'])
        self.assertFalse(tests[9]['check'])
        self.assertFalse(tests[10]['check'])

    def test_user_has_logged_in(self):
        _filter = ct_models.LastLoginfilter.objects.create(name="login Test",
                                                           description="Something to tell user",
                                                           days_since_login=30)

        c1 = ct_models.CharacterAudit.objects.filter(
            character__character_ownership__user_id=1)
        c1.update(last_known_login=timezone.now() -
                  timedelta(days=60), last_update_login=timezone.now())
        c2 = ct_models.CharacterAudit.objects.filter(
            character__character_ownership__user_id=2)
        c2.update(last_known_login=timezone.now() -
                  timedelta(days=15), last_update_login=timezone.now())

        users = []
        for user in ct_models.CharacterAudit.objects.all():
            users.append(user.character.character_ownership.user.id)

        tests = _filter.audit_filter(User.objects.filter(id__in=users))
        self.assertFalse(tests[1]['check'])
        self.assertTrue(tests[2]['check'])
        self.assertFalse(tests[3]['check'])
        self.assertFalse(tests[4]['check'])
        self.assertFalse(tests[5]['check'])
        self.assertFalse(tests[6]['check'])
        self.assertFalse(tests[7]['check'])
        self.assertFalse(tests[8]['check'])
        self.assertFalse(tests[9]['check'])
        self.assertFalse(tests[10]['check'])

    def test_legacy_user_has_logged_in(self):
        _filter = ct_models.LastLoginfilter.objects.create(name="login Test",
                                                           description="Something to tell user",
                                                           days_since_login=30)

        c1 = ct_models.CharacterAudit.objects.filter(
            character__character_ownership__user_id=1)
        c1.update(last_known_login=timezone.now() -
                  timedelta(days=60), last_update_login=timezone.now())
        c2 = ct_models.CharacterAudit.objects.filter(
            character__character_ownership__user_id=2)
        c2.update(last_known_login=timezone.now() -
                  timedelta(days=15), last_update_login=timezone.now())

        users = []
        for user in ct_models.CharacterAudit.objects.all():
            users.append(user.character.character_ownership.user.id)

        tests = {u: {"check": _filter.process_filter(
            User.objects.get(id=u))} for u in users}
        print(tests)
        self.assertFalse(tests[1]['check'])
        self.assertTrue(tests[2]['check'])
        self.assertFalse(tests[3]['check'])
        self.assertFalse(tests[4]['check'])
        self.assertFalse(tests[5]['check'])
        self.assertFalse(tests[6]['check'])
        self.assertFalse(tests[7]['check'])
        self.assertFalse(tests[8]['check'])
        self.assertFalse(tests[9]['check'])
        self.assertFalse(tests[10]['check'])

    def test_user_has_logged_in_no_data_pass(self):
        _filter = ct_models.LastLoginfilter.objects.create(name="login Test",
                                                           description="Something to tell user",
                                                           days_since_login=30,
                                                           no_data_pass=True)

        c1 = ct_models.CharacterAudit.objects.filter(
            character__character_ownership__user_id=1)
        c1.update(last_known_login=timezone.now() -
                  timedelta(days=60), last_update_login=timezone.now())
        c2 = ct_models.CharacterAudit.objects.filter(
            character__character_ownership__user_id=2)
        c2.update(last_known_login=timezone.now() -
                  timedelta(days=15), last_update_login=timezone.now())

        users = []
        for user in ct_models.CharacterAudit.objects.all():
            users.append(user.character.character_ownership.user.id)

        tests = _filter.audit_filter(User.objects.filter(id__in=users))
        self.assertFalse(tests[1]['check'])
        self.assertTrue(tests[2]['check'])
        self.assertTrue(tests[3]['check'])
        self.assertTrue(tests[4]['check'])
        self.assertTrue(tests[5]['check'])
        self.assertTrue(tests[6]['check'])
        self.assertTrue(tests[7]['check'])
        self.assertTrue(tests[8]['check'])
        self.assertTrue(tests[9]['check'])
        self.assertTrue(tests[10]['check'])

    def test_legacy_user_has_logged_in_no_data_pass(self):
        _filter = ct_models.LastLoginfilter.objects.create(name="login Test",
                                                           description="Something to tell user",
                                                           days_since_login=30,
                                                           no_data_pass=True)

        c1 = ct_models.CharacterAudit.objects.filter(
            character__character_ownership__user_id=1)
        c1.update(last_known_login=timezone.now() -
                  timedelta(days=60), last_update_login=timezone.now())
        c2 = ct_models.CharacterAudit.objects.filter(
            character__character_ownership__user_id=2)
        c2.update(last_known_login=timezone.now() -
                  timedelta(days=15), last_update_login=timezone.now())

        users = []
        for user in ct_models.CharacterAudit.objects.all():
            users.append(user.character.character_ownership.user.id)

        tests = {u: {"check": _filter.process_filter(
            User.objects.get(id=u))} for u in users}
        print(tests)

        self.assertFalse(tests[1]['check'])
        self.assertTrue(tests[2]['check'])
        self.assertTrue(tests[3]['check'])
        self.assertTrue(tests[4]['check'])
        self.assertTrue(tests[5]['check'])
        self.assertTrue(tests[6]['check'])
        self.assertTrue(tests[7]['check'])
        self.assertTrue(tests[8]['check'])
        self.assertTrue(tests[9]['check'])
        self.assertTrue(tests[10]['check'])

    def test_user_has_logged_in_main_only(self):
        _filter = ct_models.LastLoginfilter.objects.create(name="login Test",
                                                           description="Something to tell user",
                                                           days_since_login=30,
                                                           main_corp_only=True)

        c1 = ct_models.CharacterAudit.objects.filter(
            id__gte=11)  # All the alts
        c1.update(last_known_login=timezone.now() -
                  timedelta(days=15), last_update_login=timezone.now())
        c2 = ct_models.CharacterAudit.objects.filter(
            character__character_ownership__user_id=2)
        c2.update(last_known_login=timezone.now() -
                  timedelta(days=15), last_update_login=timezone.now())

        print(c1)
        print(c2)
        users = []
        for user in ct_models.CharacterAudit.objects.all():
            users.append(user.character.character_ownership.user.id)

        tests = _filter.audit_filter(User.objects.filter(id__in=users))
        self.assertFalse(tests[1]['check'])
        self.assertTrue(tests[2]['check'])
        self.assertFalse(tests[3]['check'])
        self.assertFalse(tests[4]['check'])
        self.assertFalse(tests[5]['check'])
        self.assertFalse(tests[6]['check'])
        self.assertFalse(tests[7]['check'])
        self.assertFalse(tests[8]['check'])
        self.assertFalse(tests[9]['check'])
        self.assertFalse(tests[10]['check'])

    def test_user_in_ship_l1_a1(self):
        _filter = ct_models.CurrentShipFilter.objects.create(name="Ship in Location Test",
                                                             description="Something to tell user")
        _filter.types.add(ct_models.EveItemType.objects.get(type_id=10))
        _filter.systems.add(
            ct_models.MapSystem.objects.get(name="Test System 1"))

        users = {}
        for user in ct_models.CharacterAudit.objects.all():
            users[user.character.character_ownership.user.id] = None

        tests = {}
        for k, u in users.items():
            tests[k] = _filter.process_filter(User.objects.get(id=k))

        # print("**********")
        # print(tests)

        self.assertTrue(tests[1])
        self.assertTrue(tests[2])
        self.assertFalse(tests[3])
        self.assertFalse(tests[4])
        self.assertFalse(tests[5])
        self.assertFalse(tests[6])
        self.assertFalse(tests[7])
        self.assertFalse(tests[8])
        self.assertFalse(tests[9])

    def test_user_in_ship_l2_a1(self):
        _filter = ct_models.CurrentShipFilter.objects.create(name="Ship in Location Test",
                                                             description="Something to tell user")
        _filter.types.add(ct_models.EveItemType.objects.get(type_id=10))
        _filter.systems.add(
            ct_models.MapSystem.objects.get(name="Test System 2"))

        users = {}
        for user in ct_models.CharacterAudit.objects.all():
            users[user.character.character_ownership.user.id] = None

        tests = {}
        for k, u in users.items():
            tests[k] = _filter.process_filter(User.objects.get(id=k))

        # print("**********")
        # print(tests)

        self.assertFalse(tests[1])
        self.assertFalse(tests[2])
        self.assertTrue(tests[3])
        self.assertFalse(tests[4])
        self.assertFalse(tests[5])
        self.assertFalse(tests[6])
        self.assertFalse(tests[7])
        self.assertFalse(tests[8])
        self.assertFalse(tests[9])

    def test_user_in_ship_a1(self):
        _filter = ct_models.CurrentShipFilter.objects.create(name="Ship in Location Test",
                                                             description="Something to tell user")
        _filter.types.add(ct_models.EveItemType.objects.get(type_id=10))
        # _filter.systems.add(
        #     ct_models.MapSystem.objects.get(name="Test System 1"))

        users = {}
        for user in ct_models.CharacterAudit.objects.all():
            users[user.character.character_ownership.user.id] = None

        tests = {}
        for k, u in users.items():
            tests[k] = _filter.process_filter(User.objects.get(id=k))

        # print("**********")
        # print(tests)

        self.assertTrue(tests[1])
        self.assertTrue(tests[2])
        self.assertTrue(tests[3])
        self.assertFalse(tests[4])
        self.assertFalse(tests[5])
        self.assertFalse(tests[6])
        self.assertFalse(tests[7])
        self.assertFalse(tests[8])
        self.assertFalse(tests[9])

    def test_user_in_ship_c2_a1(self):
        _filter = ct_models.CurrentShipFilter.objects.create(name="Ship in Location Test",
                                                             description="Something to tell user")
        _filter.types.add(ct_models.EveItemType.objects.get(type_id=10))
        _filter.constellations.add(
            ct_models.MapConstellation.objects.get(name="Test Constellation 2"))

        users = {}
        for user in ct_models.CharacterAudit.objects.all():
            users[user.character.character_ownership.user.id] = None

        tests = {}
        for k, u in users.items():
            tests[k] = _filter.process_filter(User.objects.get(id=k))

        # print("**********")
        # print(tests)

        self.assertFalse(tests[1])
        self.assertFalse(tests[2])
        self.assertTrue(tests[3])
        self.assertFalse(tests[4])
        self.assertFalse(tests[5])
        self.assertFalse(tests[6])
        self.assertFalse(tests[7])
        self.assertFalse(tests[8])
        self.assertFalse(tests[9])

    def test_user_in_ship_r3_a2(self):
        _filter = ct_models.CurrentShipFilter.objects.create(name="Ship in Location Test",
                                                             description="Something to tell user")
        _filter.types.add(ct_models.EveItemType.objects.get(type_id=11))
        _filter.regions.add(
            ct_models.MapRegion.objects.get(name="Test region 3"))

        users = {}
        for user in ct_models.CharacterAudit.objects.all():
            users[user.character.character_ownership.user.id] = None

        tests = {}
        for k, u in users.items():
            tests[k] = _filter.process_filter(User.objects.get(id=k))

        # print("**********")
        # print(tests)

        self.assertFalse(tests[1])
        self.assertFalse(tests[2])
        self.assertFalse(tests[3])
        self.assertFalse(tests[4])
        self.assertTrue(tests[5])
        self.assertTrue(tests[6])
        self.assertFalse(tests[7])
        self.assertFalse(tests[8])
        self.assertFalse(tests[9])

    def test_user_in_ship_r3_a3(self):
        _filter = ct_models.CurrentShipFilter.objects.create(name="Ship in Location Test",
                                                             description="Something to tell user")
        _filter.types.add(ct_models.EveItemType.objects.get(type_id=12))
        _filter.regions.add(
            ct_models.MapRegion.objects.get(name="Test region 3"))

        users = {}
        for user in ct_models.CharacterAudit.objects.all():
            users[user.character.character_ownership.user.id] = None

        tests = {}
        for k, u in users.items():
            tests[k] = _filter.process_filter(User.objects.get(id=k))

        # print("**********")
        # print(tests)

        self.assertFalse(tests[1])
        self.assertFalse(tests[2])
        self.assertFalse(tests[3])
        self.assertFalse(tests[4])
        self.assertFalse(tests[5])
        self.assertFalse(tests[6])
        self.assertFalse(tests[7])
        self.assertFalse(tests[8])
        self.assertFalse(tests[9])



from allianceauth import hooks
from allianceauth.services.hooks import MenuItemHook, UrlHook

from . import app_settings, models, urls


class MemberAudit(MenuItemHook):
    def __init__(self):

        MenuItemHook.__init__(self,
                              app_settings.CORPTOOLS_APP_NAME,
                              'far fa-eye fa-fw',
                              'corptools:react',
                              navactive=['corptools:react', 'corptools:view'])

    def render(self, request):
        if request.user.has_perm('corptools.view_characteraudit'):
            chars = request.user.character_ownerships.all(

            ).select_related('character__characteraudit')

            inactive_count = 0
            for c in chars:
                try:
                    if not c.character.characteraudit.is_active():
                        inactive_count += 1
                except models.CharacterAudit.DoesNotExist:
                    inactive_count += 1
            if inactive_count:
                self.count = inactive_count
            return MenuItemHook.render(self, request)
        return ''


class CorporationAudit(MenuItemHook):
    def __init__(self):

        MenuItemHook.__init__(self,
                              "Corporation Audit",
                              'far fa-building fa-fw',
                              'corptools:corp_react',
                              navactive=['corptools:corp_react'])

    def render(self, request):
        if (request.user.has_perm('corptools.own_corp_manager')
            or request.user.has_perm('corptools.alliance_corp_manager')
            or request.user.has_perm('corptools.state_corp_manager')
            or request.user.has_perm('corptools.global_corp_manager')
            or request.user.has_perm('corptools.holding_corp_structures')
            or request.user.has_perm('corptools.holding_corp_assets')
                or request.user.has_perm('corptools.holding_corp_wallets')):
            return MenuItemHook.render(self, request)
        return ''


@hooks.register('menu_item_hook')
def register_menu():
    return MemberAudit()


@hooks.register('menu_item_hook')
def register_corp():
    return CorporationAudit()


@hooks.register('url_hook')
def register_url():
    return UrlHook(urls, 'corptools', r'^audit/')


@hooks.register("secure_group_filters")
def filters():
    # Hey nerd, if you create a filter BEFORE its registered here. It wont appear until you edit and save it.
    return [
        models.AssetsFilter,
        models.LastLoginfilter,
        models.FullyLoadedFilter,
        models.Skillfilter,
        models.TimeInCorpFilter,
        models.Rolefilter,
        models.Titlefilter,
        models.HighestSPFilter,
        models.CurrentShipFilter,
        models.HomeStationFilter,
        models.JumpCloneFilter
    ]


@hooks.register('discord_cogs_hook')
def register_cogs():
    return app_settings.CORPTOOLS_DISCORD_BOT_COGS



# Generated by Django 4.0.10 on 2023-05-02 12:02

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0084_characteraudit_last_update_loyaltypoints_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='characteraudit',
            name='last_update_mining',
            field=models.DateTimeField(blank=True, default=None, null=True),
        ),
        migrations.CreateModel(
            name='CharacterMiningLedger',
            fields=[
                ('id', models.CharField(max_length=50,
                 primary_key=True, serialize=False)),
                ('date', models.DateField()),
                ('quantity', models.IntegerField()),
                ('character', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE, to='corptools.characteraudit')),
                ('system', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE, to='corptools.mapsystem')),
                ('type_name', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE, to='corptools.eveitemtype')),
            ],
        ),
    ]



# Generated by Django 2.2.12 on 2020-08-04 09:06

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0019_implant_jumpclone'),
    ]

    operations = [
        migrations.CreateModel(
            name='Clone',
            fields=[
                ('id', models.AutoField(auto_created=True,
                                        primary_key=True, serialize=False, verbose_name='ID')),
                ('last_clone_jump_date', models.DateTimeField(
                    default=None, null=True)),
                ('last_station_change_date',
                 models.DateTimeField(default=None, null=True)),
                ('location_id', models.BigIntegerField()),
                ('location_type', models.CharField(choices=[
                 ('station', 'station'), ('structure', 'structure')], max_length=9)),
                ('character', models.OneToOneField(
                    on_delete=django.db.models.deletion.CASCADE, to='corptools.CharacterAudit')),
                ('location_name', models.ForeignKey(default=None, null=True,
                                                    on_delete=django.db.models.deletion.SET_NULL, to='corptools.EveLocation')),
            ],
        ),
    ]



# Generated by Django 4.0.10 on 2023-11-25 09:25

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0098_migrate_role_filters'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='rolefilter',
            name='alliance_filter',
        ),
        migrations.RemoveField(
            model_name='rolefilter',
            name='corp_filter',
        ),
    ]



import logging
from typing import List

from ninja import NinjaAPI

from django.http import HttpResponse

from allianceauth.services.hooks import get_extension_logger

from corptools import models
from corptools.api import schema
from corptools.api.helpers import round_or_null

logger = get_extension_logger(__name__)


class StructureApiEndpoints:

    tags = ["Structures"]

    def __init__(self, api: NinjaAPI):
        @api.get(
            "corp/structures",
            response={200: List[schema.Structure], 403: str},
            tags=self.tags
        )
        def get_visible_structures(request):
            perms = (
                request.user.has_perm('corptools.own_corp_manager')
                | request.user.has_perm('corptools.alliance_corp_manager')
                | request.user.has_perm('corptools.state_corp_manager')
                | request.user.has_perm('corptools.global_corp_manager')
                | request.user.has_perm('corptools.holding_corp_structures')
            )

            if not perms:
                logging.error(
                    f"Permission Denied for {request.user} to view structures!")
                return 403, "Permission Denied!"

            output = []
            for s in models.Structure.get_visible(request.user).select_related(
                'type_name', "corporation__corporation", "system_name"
            ).prefetch_related('structureservice_set'):
                _ss = list()
                for __s in s.structureservice_set.all():
                    _ss.append({
                        "name": __s.name,
                        "state": __s.state
                    })
                _s = {
                    "id": s.structure_id,
                    "owner": s.corporation.corporation,
                    "name": s.name,
                    "type": {"id": s.type_id,
                             "name": s.type_name.name},
                    "services": _ss,
                    "location": {"id": s.system_name.system_id,
                                 "name": s.system_name.name},
                    "fuel_expiry": s.fuel_expires,
                    "state": s.state,
                    "state_expiry": s.state_timer_end
                }
                output.append(_s)
            return list(output)

        @api.get(
            "corp/structures/{structure_id}",
            response={200: List[schema.FittingItem], 404: str, 403: str},
            tags=self.tags
        )
        def get_corporation_structure_fitting(request, corporation_id, structure_id):
            output = []
            return 200, output

        @api.get(
            "corp/pocos",
            response={200: List, 403: str},
            tags=self.tags
        )
        def get_visible_pocos(request):
            perms = (
                request.user.has_perm('corptools.own_corp_manager')
                | request.user.has_perm('corptools.alliance_corp_manager')
                | request.user.has_perm('corptools.state_corp_manager')
                | request.user.has_perm('corptools.global_corp_manager')
                | request.user.has_perm('corptools.holding_corp_structures')
            )

            if not perms:
                logging.error(
                    f"Permission Denied for {request.user} to view pocos!")
                return 403, "Permission Denied!"

            output = []
            for s in models.Poco.get_visible(request.user).select_related(
                "corporation__corporation", "system_name", "system_name__constellation", "system_name__constellation__region", "planet"
            ):
                _s = {
                    "id": s.office_id,
                    "owner": {
                        "corporation_name": s.corporation.corporation.corporation_name,
                        "corporation_id": s.corporation.corporation.corporation_id
                    },
                    "name": s.name,
                    "location": {"id": s.planet.planet_id,
                                 "name": s.planet.name,
                                 "region": s.system_name.constellation.region.name,
                                 "constellation": s.system_name.constellation.name},
                    "standing_level": s.standing_level,
                    "alliance_tax_rate": round_or_null(s.alliance_tax_rate),
                    "corporation_tax_rate": round_or_null(s.corporation_tax_rate),
                    "terrible_standing_tax_rate": round_or_null(s.terrible_standing_tax_rate),
                    "bad_standing_tax_rate": round_or_null(s.bad_standing_tax_rate),
                    "neutral_standing_tax_rate": round_or_null(s.neutral_standing_tax_rate),
                    "good_standing_tax_rate": round_or_null(s.good_standing_tax_rate),
                    "excellent_standing_tax_rate": round_or_null(s.excellent_standing_tax_rate),
                    "reinforce_exit_end": s.reinforce_exit_end,
                    "reinforce_exit_start": s.reinforce_exit_start,
                    "allow_access_with_standings": s.allow_access_with_standings,
                    "allow_alliance_access": s.allow_alliance_access,
                }
                output.append(_s)
            return list(output)

        @api.get(
            "extra/jb/export",
            response={200: str, 403: str},
            tags=["Extras"]
        )
        def get_jb_link(request):
            output = []
            for s in models.Structure.objects.filter(
                type_id=35841
            ):
                id = s.structure_id
                split = s.name.split(" » ")

                from_sys = split[0]
                to_sys = split[1].split(" - ")[0]

                output.append(f"{id} {from_sys} --> {to_sys}")
            return HttpResponse("\n".join(output), content_type="text/plain")



from django.core.management.base import BaseCommand

from corptools.tasks import update_all_raw_minerals


class Command(BaseCommand):
    help = 'Update or Populate EVE Item prices for cat_id 4'

    def handle(self, *args, **options):
        self.stdout.write("Getting Prices")
        return update_all_raw_minerals()



# Generated by Django 3.2.5 on 2021-08-08 03:49

import django.db.models.deletion
from django.db import migrations, models
from django.db.models import Count


def remove_dupes(apps, schema_editor):
    # We can't import the Person model directly as it may be a newer
    # version than this migration expects. We use the historical version.
    CharacterRoles = apps.get_model('corptools', 'CharacterRoles')
    ids = CharacterRoles.objects.values('character').annotate(
        Count('id')).order_by().filter(id__count__gt=1)
    for id in ids:
        _id = CharacterRoles.objects.filter(
            character_id=id['character']).last().pk
        CharacterRoles.objects.filter(
            character_id=id['character']).exclude(pk=_id).delete()


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0054_auto_20210808_0138'),
    ]

    operations = [
        migrations.RunPython(remove_dupes),
        migrations.AlterField(
            model_name='characterroles',
            name='character',
            field=models.OneToOneField(
                on_delete=django.db.models.deletion.CASCADE, to='corptools.characteraudit'),
        ),
    ]



from .asset_pinger import AssetPingApiEndpoints
from .clone_deaths import CloneDeathApiEndpoints
from .fittings import FittingsApiEndpoints


def setup(api):
    FittingsApiEndpoints(api)
    AssetPingApiEndpoints(api)
    CloneDeathApiEndpoints(api)



from django_celery_beat.models import CrontabSchedule, PeriodicTask

from django.core.management.base import BaseCommand

from corptools.tasks import update_or_create_map


class Command(BaseCommand):
    help = 'Bootstrap the CorpTools Module'

    def add_arguments(self, parser):
        parser.add_argument('--inline', action='store_true',
                            help='Run update in this Console not via Celery')

    def handle(self, *args, **options):
        self.stdout.write("Configuring Tasks!")
        schedule_char, _ = CrontabSchedule.objects.get_or_create(minute='15,45',
                                                                 hour='*',
                                                                 day_of_week='*',
                                                                 day_of_month='*',
                                                                 month_of_year='*',
                                                                 timezone='UTC'
                                                                 )

        schedule_corp, _ = CrontabSchedule.objects.get_or_create(minute='30',
                                                                 hour='*',
                                                                 day_of_week='*',
                                                                 day_of_month='*',
                                                                 month_of_year='*',
                                                                 timezone='UTC'
                                                                 )

        PeriodicTask.objects.update_or_create(
            task='corptools.tasks.update_subset_of_characters',
            defaults={
                'crontab': schedule_char,
                'name': 'Character Audit Rolling Update',
                'enabled': True
            }
        )

        PeriodicTask.objects.update_or_create(
            task='corptools.tasks.update_all_corps',
            defaults={
                'crontab': schedule_corp,
                'name': 'Corporation Audit Update',
                'enabled': True
            }
        )

        self.stdout.write("Populating DB models!")
        if options['inline']:
            self.stdout.write("Running Tasks inline this may take some time!")
            self.stdout.write("Starting Map Update")
            update_or_create_map()
            self.stdout.write("Done Tasks!")
        else:
            self.stdout.write("Sending Tasks to celery for processing!")
            self.stdout.write("Sending Map Update Task")
            update_or_create_map.apply_async(priority=6)
            self.stdout.write("Tasks Queued!")



# Generated by Django 2.2.9 on 2020-05-11 08:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0009_auto_20200511_0816'),
    ]

    operations = [
        migrations.AlterField(
            model_name='characterasset',
            name='item_id',
            field=models.BigIntegerField(),
        ),
        migrations.AlterField(
            model_name='corpasset',
            name='item_id',
            field=models.BigIntegerField(),
        ),
    ]



# Generated by Django 2.2.9 on 2020-05-07 11:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0003_auto_20200507_1115'),
    ]

    operations = [
        migrations.AlterField(
            model_name='mapsystem',
            name='security_class',
            field=models.CharField(default=None, max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='mapsystem',
            name='star_id',
            field=models.IntegerField(default=None, null=True),
        ),
    ]



# Generated by Django 3.2.5 on 2021-07-19 10:39

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0051_auto_20210713_0557'),
    ]

    operations = [
        migrations.CreateModel(
            name='CorporationContactLabel',
            fields=[
                ('id', models.BigIntegerField(primary_key=True, serialize=False)),
                ('label_id', models.BigIntegerField()),
                ('label_name', models.CharField(max_length=255)),
                ('corporation', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE, to='corptools.corporationaudit')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CorporationContact',
            fields=[
                ('id', models.BigIntegerField(primary_key=True, serialize=False)),
                ('contact_id', models.BigIntegerField()),
                ('contact_type', models.CharField(max_length=255)),
                ('standing', models.DecimalField(decimal_places=2, max_digits=4)),
                ('watched', models.BooleanField(default=False)),
                ('contact_name', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE, to='corptools.evename')),
                ('corporation', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE, to='corptools.corporationaudit')),
                ('labels', models.ManyToManyField(
                    to='corptools.CorporationContactLabel')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CharacterContactLabel',
            fields=[
                ('id', models.BigIntegerField(primary_key=True, serialize=False)),
                ('label_id', models.BigIntegerField()),
                ('label_name', models.CharField(max_length=255)),
                ('character', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE, to='corptools.characteraudit')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CharacterContact',
            fields=[
                ('id', models.BigIntegerField(primary_key=True, serialize=False)),
                ('contact_id', models.BigIntegerField()),
                ('contact_type', models.CharField(max_length=255)),
                ('standing', models.DecimalField(decimal_places=2, max_digits=4)),
                ('blocked', models.BooleanField(default=False)),
                ('watched', models.BooleanField(default=False)),
                ('character', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE, to='corptools.characteraudit')),
                ('contact_name', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE, to='corptools.evename')),
                ('labels', models.ManyToManyField(
                    to='corptools.CharacterContactLabel')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



import logging

from discord.colour import Color
from discord.embeds import Embed
from discord.ext import commands

from corptools.models import MapJumpBridge, MapSystem
from corptools.providers import routes

logger = logging.getLogger(__name__)


class Routes(commands.Cog):
    """
    Route!
    """

    def __init__(self, bot):
        self.bot = bot

    @commands.command(pass_context=True)
    async def route(self, ctx):
        """
        Find route in eve with JB's
        """
        input_names = ctx.message.content[7:].split(":")
        start = MapSystem.objects.get(name=input_names[0])
        end = MapSystem.objects.get(name=input_names[1])

        message = routes.route(start.system_id, end.system_id)

        dotlan_url = "https://evemaps.dotlan.net/route/{}".format(
            message.get("dotlan"))
        embed = Embed(title=f"{start.name} to {end.name}")
        embed.colour = Color.blue()
        embed.description = "Shortest Route is: {} Jumps\n\n{}".format(
            message.get("length"), message.get("path_message"))
        embed.add_field(
            name="Dotlan", value=f"[Route Link]({dotlan_url})"
        )

        return await ctx.send(embed=embed)

    @commands.command(pass_context=True)
    async def jumpbridges(self, ctx):
        """
        List all known Jumpbridges's
        """

        embed = Embed(title="Known Jump Bridges")
        embed.colour = Color.blue()
        embed.description = "These do not auto populate. Please advise admins of ommisions/errors!\n\n"

        jbs = MapJumpBridge.objects.all().select_related(
            'from_solar_system', 'to_solar_system', 'owner')
        for jb in jbs:
            embed.description += f"`{jb.from_solar_system.name}` > `{jb.to_solar_system}` [{jb.owner.name}]\n"
        return await ctx.send(embed=embed)


def setup(bot):
    bot.add_cog(Routes(bot))



from ..models import CorporationAudit
from . import CorptoolsTestCase


class TestCorptoolsCorpAccessPerms(CorptoolsTestCase):
    def test_no_perms_get_self_u1(self):
        cs = CorporationAudit.objects.visible_to(self.user1)
        self.assertNotIn(self.cp1, cs)
        self.assertNotIn(self.cp2, cs)
        self.assertNotIn(self.cp3, cs)
        self.assertNotIn(self.cp4, cs)

    def test_no_perms_get_self_u2(self):
        cs = CorporationAudit.objects.visible_to(self.user2)
        self.assertNotIn(self.cp1, cs)
        self.assertNotIn(self.cp2, cs)
        self.assertNotIn(self.cp3, cs)
        self.assertNotIn(self.cp4, cs)

    def test_no_perms_get_self_u3(self):
        cs = CorporationAudit.objects.visible_to(self.user2)
        self.assertNotIn(self.cp1, cs)
        self.assertNotIn(self.cp2, cs)
        self.assertNotIn(self.cp3, cs)
        self.assertNotIn(self.cp4, cs)

    def test_get_corp_u1(self):
        self.user1.user_permissions.add(self.own_corp_manager)
        self.user1.refresh_from_db()
        cs = CorporationAudit.objects.visible_to(self.user1)
        self.assertIn(self.cp1, cs)
        self.assertNotIn(self.cp2, cs)
        self.assertNotIn(self.cp3, cs)
        self.assertNotIn(self.cp4, cs)

    def test_get_corp_u2(self):
        self.user2.user_permissions.add(self.own_corp_manager)
        self.user2.refresh_from_db()
        cs = CorporationAudit.objects.visible_to(self.user2)
        self.assertNotIn(self.cp1, cs)
        self.assertIn(self.cp2, cs)
        self.assertNotIn(self.cp3, cs)
        self.assertNotIn(self.cp4, cs)

    def test_get_corp_u3(self):
        self.user3.user_permissions.add(self.own_corp_manager)
        self.user3.refresh_from_db()
        cs = CorporationAudit.objects.visible_to(self.user3)
        self.assertNotIn(self.cp1, cs)
        self.assertNotIn(self.cp2, cs)
        self.assertIn(self.cp3, cs)
        self.assertNotIn(self.cp4, cs)

    def test_get_alliance_u1(self):
        self.user1.user_permissions.add(self.alliance_corp_manager)
        self.user1.refresh_from_db()
        cs = CorporationAudit.objects.visible_to(self.user1)
        self.assertIn(self.cp1, cs)
        self.assertNotIn(self.cp2, cs)
        self.assertNotIn(self.cp3, cs)
        self.assertNotIn(self.cp4, cs)

    def test_get_alliance_u2(self):
        self.user2.user_permissions.add(self.alliance_corp_manager)
        self.user2.refresh_from_db()
        cs = CorporationAudit.objects.visible_to(self.user2)
        self.assertNotIn(self.cp1, cs)
        self.assertIn(self.cp2, cs)
        self.assertNotIn(self.cp3, cs)
        self.assertNotIn(self.cp4, cs)

    def test_get_alliance_u3(self):
        self.user3.user_permissions.add(self.alliance_corp_manager)
        self.user3.refresh_from_db()
        cs = CorporationAudit.objects.visible_to(self.user3)
        self.assertNotIn(self.cp1, cs)
        self.assertNotIn(self.cp2, cs)
        self.assertIn(self.cp3, cs)
        self.assertIn(self.cp4, cs)

    def test_global_perms_u1(self):
        self.user1.user_permissions.add(self.global_corp_manager)
        self.user1.refresh_from_db()
        cs = CorporationAudit.objects.visible_to(self.user1)
        self.assertIn(self.cp1, cs)
        self.assertIn(self.cp2, cs)
        self.assertIn(self.cp3, cs)
        self.assertIn(self.cp4, cs)

    def test_global_perms_u2(self):
        self.user2.user_permissions.add(self.global_corp_manager)
        self.user2.refresh_from_db()
        cs = CorporationAudit.objects.visible_to(self.user2)
        self.assertIn(self.cp1, cs)
        self.assertIn(self.cp2, cs)
        self.assertIn(self.cp3, cs)
        self.assertIn(self.cp4, cs)

    def test_global_perms_u3(self):
        self.user3.user_permissions.add(self.global_corp_manager)
        self.user3.refresh_from_db()
        cs = CorporationAudit.objects.visible_to(self.user3)
        self.assertIn(self.cp1, cs)
        self.assertIn(self.cp2, cs)
        self.assertIn(self.cp3, cs)
        self.assertIn(self.cp4, cs)



import re

import networkx as nx

from django.utils import timezone

from esi.clients import EsiClientProvider

from .task_helpers.skill_helpers import SkillListCache


class CorpToolsESIClient(EsiClientProvider):

    # TODO create provider dummy classes for use here to not have to deal with ORM model bullshit and maybe be more async?.

    @staticmethod
    def chunk_ids(lo, n=750):
        for i in range(0, len(lo), n):
            yield lo[i:i + n]

    def _get_category(self, category_id, updates=False):
        from corptools.models import EveItemCategory

        _category = self.client.Universe.get_universe_categories_category_id(
            category_id=category_id).result()
        groups = _category.get('groups', [])
        category = EveItemCategory(
            category_id=category_id, name=_category.get('name'))

        if updates is not False:
            if category_id in updates:
                return category, False, groups
            else:
                return category, True, groups
        # category.save()
        return category

    def _get_group(self, group_id, updates=False):
        from corptools.models import EveItemGroup

        _group = self.client.Universe.get_universe_groups_group_id(
            group_id=group_id).result()
        eve_types = _group.get('types', [])
        group = EveItemGroup(group_id=group_id, category_id=_group.get(
            'category_id'), name=_group.get('name'))

        if updates is not False:
            if group_id in updates:
                return group, False, eve_types
            else:
                return group, True, eve_types
        # group.save()
        return group

    def _get_eve_type(self, type_id, updates=False):
        from corptools.models import EveItemDogmaAttribute, EveItemType

        eve_type = self.client.Universe.get_universe_types_type_id(
            type_id=type_id).result()
        dogma = []
        if eve_type.get('dogma_attributes'):
            for d_at in eve_type.get('dogma_attributes', []):
                d_at_temp = EveItemDogmaAttribute(eve_type_id=type_id,
                                                  attribute_id=d_at.get(
                                                      "attribute_id"),
                                                  value=d_at.get('value'))
                dogma.append(d_at_temp)

        eve_type = EveItemType(type_id=type_id,
                               group_id=eve_type.get('group_id'),
                               name=eve_type.get('name'),
                               description=eve_type.get('description'),
                               mass=eve_type.get('mass'),
                               packaged_volume=eve_type.get('packaged_volume'),
                               portion_size=eve_type.get('portion_size'),
                               volume=eve_type.get('volume'),
                               published=eve_type.get('published'),
                               radius=eve_type.get('radius')
                               )
        if updates is not False:
            if type_id in updates:
                return eve_type, False, dogma
            else:
                return eve_type, True, dogma
        # eve_type.save()
        return eve_type, dogma

    def _get_region(self, region_id, updates):
        from corptools.models import MapRegion

        region = self.client.Universe.get_universe_regions_region_id(
            region_id=region_id).result()
        constelations = region.get('constellations', [])
        region = MapRegion(region_id=region_id,
                           name=region.get('name'),
                           description=region.get('description'))

        if region_id in updates:
            return region, False, constelations
        else:
            return False, region, constelations

    def _get_constellation(self, constellation_id, updates):
        from corptools.models import MapConstellation

        constellation = self.client.Universe.get_universe_constellations_constellation_id(
            constellation_id=constellation_id).result()
        systems = constellation.get('systems', [])
        constellation = MapConstellation(
            constellation_id=constellation_id,
            region_id=constellation.get('region_id'),
            name=constellation.get('name'))

        if constellation_id in updates:
            return constellation, False, systems
        else:
            return False, constellation, systems

    def _get_system(self, system_id, updates):
        from corptools.models import MapSystem

        system = self.client.Universe.get_universe_systems_system_id(
            system_id=system_id).result()
        gates = system.get('stargates', None)
        system = MapSystem(system_id=system_id,
                           constellation_id=system.get('constellation_id'),
                           name=system.get('name'),
                           security_class=system.get('security_class', None),
                           security_status=system.get('security_status'),
                           star_id=system.get('star_id', None),
                           x=system.get('position', {}).get('x'),
                           y=system.get('position', {}).get('y'),
                           z=system.get('position', {}).get('z'),
                           )
        if system_id in updates:
            return system, False, gates
        else:
            return False, system, gates

    def _get_moon(self, moon_id, updates):
        from corptools.models import MapSystemMoon

        moon = self.client.Universe.get_universe_moons_moon_id(
            moon_id=moon_id).result()
        moon = MapSystemMoon(moon_id=moon_id,
                             name=moon.get('name'),
                             system_id=moon.get('system_id', None),
                             x=moon.get('position', {}).get('x'),
                             y=moon.get('position', {}).get('y'),
                             z=moon.get('position', {}).get('z'),
                             )
        if updates is not False:
            if moon_id in updates:
                return moon, False
            else:
                return False, moon
        else:
            return moon

    def _get_planet(self, planet_id, updates):
        from corptools.models import MapSystemPlanet

        planet = self.client.Universe.get_universe_planets_planet_id(
            planet_id=planet_id).result()
        planet = MapSystemPlanet(planet_id=planet_id,
                                 name=planet.get('name'),
                                 system_id=planet.get('system_id', None),
                                 x=planet.get('position', {}).get('x'),
                                 y=planet.get('position', {}).get('y'),
                                 z=planet.get('position', {}).get('z'),
                                 )
        if updates is not False:
            if planet_id in updates:
                return planet, False
            else:
                return False, planet
        else:
            return planet

    def _get_stargate(self, gate_id):
        gate = self.client.Universe.get_universe_stargates_stargate_id(
            stargate_id=gate_id).result()
        return gate['system_id'], gate['destination']['system_id']


class EveRouter():
    def __init__(self):
        self.G = nx.Graph()
        self.last_update = None
        self.bridges = {}

    def bulid_graph(self):
        self.G.clear()
        # logger.debug("Graph cleared.")
        from .models import MapJumpBridge, MapSystem, MapSystemGate

        systems = MapSystem.objects.values_list('system_id', flat=True)
        gates = MapSystemGate.objects.values_list(
            'from_solar_system_id', 'to_solar_system_id')
        bridges_query = MapJumpBridge.objects.values_list(
            'from_solar_system_id', 'to_solar_system_id', 'structure_id')
        bridges = []
        bridge_ids = {}
        for b in bridges_query:
            bridge_ids[b[0]] = b[2]
            bridges.append([b[0], b[1]])
        self.G.add_nodes_from(systems)
        self.G.add_edges_from(gates, type="gate")
        self.G.add_edges_from(bridges, type="bridge")
        self.bridges = bridge_ids
        self.last_update = timezone.now()

    def route(self, source_id, destination_id):
        from .models import MapJumpBridge, MapSystem

        # logger.debug("----------")
        # logger.debug(f"Source: {source_id}")
        # logger.debug(f"Destination: {destination_id}")

        if not self.last_update:
            self.bulid_graph()
        else:
            try:
                last_update = MapJumpBridge.objects.latest("updated").updated
                if last_update > self.last_update:
                    self.bulid_graph()
            except MapJumpBridge.DoesNotExist:
                pass

        path = nx.shortest_path(self.G, source_id, destination_id)
        path_length = len(path) - 1
        systems = MapSystem.objects.filter(system_id__in=path)
        system_map = {}

        for a in systems:
            system_map[a.system_id] = a.name

        output = {}
        for i, p in enumerate(path):
            output[i] = system_map[p]

        dotlan_path = output[0].replace(" ", "_")
        message_path = "`" + output[0].replace(" ", "_") + "`"
        esi_points = []
        for i in range(len(path) - 1):
            if self.G.get_edge_data(path[i], path[i + 1])['type'] == 'bridge':
                message_path += "  >B>  `" + output[i + 1] + "`"
                bridged_path = '::' + output[i + 1].replace(" ", "_")
                dotlan_path += bridged_path
                esi_points.append(self.bridges[path[i]])
            else:
                message_path += "  >G>  `" + output[i + 1] + "`"
                gated_path = ':' + output[i + 1].replace(" ", "_")
                dotlan_path += gated_path
        dotlan_path = re.sub(r'(?<!:)(:[^:\s]+)(?=:)(?!::)', '', dotlan_path)
        esi_points.append(path[len(path) - 1])
        result = {'path': output, 'esi': esi_points, 'path_message': message_path,
                  'dotlan': dotlan_path, 'length': path_length}

        return result


esi = CorpToolsESIClient()
routes = EveRouter()
skills = SkillListCache()



# Generated by Django 3.2.5 on 2021-07-19 11:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0052_charactercontact_charactercontactlabel_corporationcontact_corporationcontactlabel'),
    ]

    operations = [
        migrations.AddField(
            model_name='characteraudit',
            name='last_update_contacts',
            field=models.DateTimeField(blank=True, default=None, null=True),
        ),
    ]
